@@ -37,6 +37,16 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 {
 	setupUi(this);
 
+	m_configlists->refreshModules();
+
+	if(m_configlists->rtAudioNames.size() == 1) {
+		QMessageBox::warning(this, tr("No Audio Modules"),
+							 tr("No real-time audio modules were found.\nMake sure OPCODE6DIR64 is set properly in your system or the configuration dialog."));
+	}
+	QHash<QString, QString> audioModNames;
+	audioModNames["pa_bl"] = "portaudio (blocking)";
+	audioModNames["pa_cb"] = "portaudio (callback)";
+	audioModNames["auhal"] = "coreaudio (auhal)";
 	foreach (QString item, m_configlists->fileTypeLongNames) {
 		FileTypeComboBox->addItem(item);
 	}
@@ -44,15 +54,20 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 		FileFormatComboBox->addItem(item);
 	}
 	foreach (QString item, m_configlists->rtAudioNames) {
-		RtModuleComboBox->addItem(item);
+		if (audioModNames.contains(item)) {
+			RtModuleComboBox->addItem(audioModNames[item], item);
+		} else {
+			RtModuleComboBox->addItem(item, item);
+		}
 	}
 	foreach (QString item, m_configlists->rtMidiNames) {
-		RtMidiModuleComboBox->addItem(item);
+		RtMidiModuleComboBox->addItem(item, item);
 	}
 	for (int i = 0; i < m_configlists->languages.size(); i++) {
 		languageComboBox->addItem(m_configlists->languages[i], QVariant(m_configlists->languageCodes[i]));
 	}
 	midiInterfaceComboBox->clear();
+	midiOutInterfaceComboBox->clear();
 
 #ifdef QCS_RTMIDI
 	try {
@@ -65,13 +80,26 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 		// Handle the exception here
 		error.printMessage();
 	}
+	try {
+		RtMidiOut midiout;
+		for (int i = 0; i < (int) midiout.getPortCount(); i++) {
+			midiOutInterfaceComboBox->addItem(QString::fromStdString(midiout.getPortName(i)), QVariant(i));
+		}
+	}
+	catch (RtError &error) {
+		// Handle the exception here
+		error.printMessage();
+	}
 #else
 	midiInterfaceComboBox->addItem(tr("No RtMidi support"));
 #endif
 
-	midiInterfaceComboBox->addItem(QString(tr("None", "No MIDI internal interface")), QVariant(9999));
+	midiInterfaceComboBox->addItem(QString(tr("None", "No MIDI In interface")), QVariant(9999));
 	int ifIndex = midiInterfaceComboBox->findData(QVariant(m_options->midiInterface));
 	midiInterfaceComboBox->setCurrentIndex(ifIndex);
+	midiOutInterfaceComboBox->addItem(QString(tr("None", "No MIDI Out interface")), QVariant(9999));
+	ifIndex = midiOutInterfaceComboBox->findData(QVariant(m_options->midiOutInterface));
+	midiOutInterfaceComboBox->setCurrentIndex(ifIndex);
 
 	themeComboBox->setCurrentIndex(themeComboBox->findText(m_options->theme));
 	fontComboBox->setCurrentIndex(fontComboBox->findText(m_options->font) );
@@ -92,6 +120,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	consoleBgColorPushButton->setPalette(palette);
 
 	tabWidthSpinBox->setValue(m_options->tabWidth);
+	tabIndentCheckBox->setChecked(m_options->tabIndents);
 	colorVariablesCheckBox->setChecked(m_options->colorVariables);
 	autoplayCheckBox->setChecked(m_options->autoPlay);
 	autoJoinCheckBox->setChecked(m_options->autoJoin);
@@ -104,6 +133,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	toolbarCheckBox->setChecked(m_options->showToolbar);
 	wrapLinesCheckBox->setChecked(m_options->wrapLines);
 	autoCompleteCheckBox->setChecked(m_options->autoComplete);
+	autoParameterModeCheckBox->setChecked(m_options->autoParameterMode);
 	widgetsCheckBox->setChecked(m_options->enableWidgets);
 	showWidgetsOnRunCheckBox->setChecked(m_options->showWidgetsOnRun);
 	showTooltipsCheckBox->setChecked(m_options->showTooltips);
@@ -163,13 +193,18 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	OutputFilenameLineEdit->setEnabled(m_options->fileOutputFilenameActive);
 	RtUseOptionsCheckBox->setChecked(m_options->rtUseOptions);
 	RtOverrideCheckBox->setChecked(m_options->rtOverrideOptions);
-	RtModuleComboBox->setCurrentIndex(RtModuleComboBox->findText(m_options->rtAudioModule));
+	int index = RtModuleComboBox->findData(m_options->rtAudioModule);
+	if (index < 0 || index >= RtModuleComboBox->count()) {
+		index = 0;
+	}
+	RtModuleComboBox->setCurrentIndex(index);
 	RtInputLineEdit->setText(m_options->rtInputDevice);
 	RtOutputLineEdit->setText(m_options->rtOutputDevice);
 	JackNameLineEdit->setText(m_options->rtJackName);
-	RtMidiModuleComboBox->setCurrentIndex(RtMidiModuleComboBox->findText(m_options->rtMidiModule));
+	RtMidiModuleComboBox->setCurrentIndex(RtMidiModuleComboBox->findData(m_options->rtMidiModule));
 	RtMidiInputLineEdit->setText(m_options->rtMidiInputDevice);
 	RtMidiOutputLineEdit->setText(m_options->rtMidiOutputDevice);
+	csoundMidiCheckBox->setChecked(m_options->useCsoundMidi);
 	simultaneousCheckBox->setChecked(m_options->simultaneousRun);
 
 	sampleFormatComboBox->setCurrentIndex(m_options->sampleFormat);
@@ -177,25 +212,20 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	CsdocdirLineEdit->setText(m_options->csdocdir);
 	OpcodedirCheckBox->setChecked(m_options->opcodedirActive);
 	OpcodedirLineEdit->setText(m_options->opcodedir);
-	OpcodedirLineEdit->setEnabled(m_options->opcodedirActive);
 	Opcodedir64CheckBox->setChecked(m_options->opcodedir64Active);
 	Opcodedir64LineEdit->setText(m_options->opcodedir64);
-	Opcodedir64LineEdit->setEnabled(m_options->opcodedir64Active);
 	Opcode6dir64CheckBox->setChecked(m_options->opcode6dir64Active);
 	Opcode6dir64LineEdit->setText(m_options->opcode6dir64);
-	Opcode6dir64LineEdit->setEnabled(m_options->opcode6dir64Active);
 	SadirCheckBox->setChecked(m_options->sadirActive);
 	SadirLineEdit->setText(m_options->sadir);
-	SadirLineEdit->setEnabled(m_options->sadirActive);
 	SsdirCheckBox->setChecked(m_options->ssdirActive);
 	SsdirLineEdit->setText(m_options->ssdir);
-	SsdirLineEdit->setEnabled(m_options->ssdirActive);
 	SfdirCheckBox->setChecked(m_options->sfdirActive);
 	SfdirLineEdit->setText(m_options->sfdir);
-	SfdirLineEdit->setEnabled(m_options->sfdirActive);
+	rawWaveCheckBox->setChecked(m_options->rawWaveActive);
+	rawWaveLineEdit->setText(m_options->rawWave);
 	IncdirCheckBox->setChecked(m_options->incdirActive);
 	IncdirLineEdit->setText(m_options->incdir);
-	IncdirLineEdit->setEnabled(m_options->incdirActive);
 	//  defaultCsdCheckBox->setChecked(m_options->defaultCsdActive);
 	//  defaultCsdLineEdit->setText(m_options->defaultCsd);
 	//  defaultCsdLineEdit->setEnabled(m_options->defaultCsdActive);
@@ -298,6 +328,7 @@ void ConfigDialog::accept()
 	m_options->consoleFont = consoleFontComboBox->currentText();
 	m_options->consoleFontPointSize = consoleFontSizeComboBox->currentText().toDouble();
 	m_options->tabWidth = tabWidthSpinBox->value();
+	m_options->tabIndents = tabIndentCheckBox->isChecked();
 	m_options->colorVariables = colorVariablesCheckBox->isChecked();
 	m_options->autoPlay = autoplayCheckBox->isChecked();
 	m_options->autoJoin = autoJoinCheckBox->isChecked();
@@ -310,6 +341,7 @@ void ConfigDialog::accept()
 	m_options->showToolbar = toolbarCheckBox->isChecked();
 	m_options->wrapLines = wrapLinesCheckBox->isChecked();
 	m_options->autoComplete = autoCompleteCheckBox->isChecked();
+	m_options->autoParameterMode = autoParameterModeCheckBox->isChecked();
 	m_options->enableWidgets = widgetsCheckBox->isChecked();
 	m_options->showWidgetsOnRun = showWidgetsOnRunCheckBox->isChecked();
 	m_options->showTooltips = showTooltipsCheckBox->isChecked();
@@ -326,6 +358,7 @@ void ConfigDialog::accept()
 	m_options->debugLiveEvents = debugLiveEventsCheckBox->isChecked();
 	m_options->consoleBufferSize = consoleBufferComboBox->itemText(consoleBufferComboBox->currentIndex()).toInt();
 	m_options->midiInterface = midiInterfaceComboBox->itemData(midiInterfaceComboBox->currentIndex()).toInt();
+	m_options->midiOutInterface = midiOutInterfaceComboBox->itemData(midiOutInterfaceComboBox->currentIndex()).toInt();
 	m_options->noMessages = noMessagesCheckBox->isChecked();
 	m_options->noBuffer = noBufferCheckBox->isChecked();
 	m_options->noPython = noPythonCheckBox->isChecked();
@@ -354,13 +387,14 @@ void ConfigDialog::accept()
 	m_options->fileOutputFilename = OutputFilenameLineEdit->text();
 	m_options->rtUseOptions = RtUseOptionsCheckBox->isChecked();
 	m_options->rtOverrideOptions = RtOverrideCheckBox->isChecked();
-	m_options->rtAudioModule = RtModuleComboBox->currentText();
+	m_options->rtAudioModule = RtModuleComboBox->itemData(RtModuleComboBox->currentIndex()).toString();
 	m_options->rtInputDevice = RtInputLineEdit->text();
 	m_options->rtOutputDevice = RtOutputLineEdit->text();
 	m_options->rtJackName = JackNameLineEdit->text();
 	m_options->rtMidiModule = RtMidiModuleComboBox->currentText();
 	m_options->rtMidiInputDevice = RtMidiInputLineEdit->text();
 	m_options->rtMidiOutputDevice = RtMidiOutputLineEdit->text();
+	m_options->useCsoundMidi = csoundMidiCheckBox->isChecked();
 	m_options->simultaneousRun = simultaneousCheckBox->isChecked();
 
 	m_options->sampleFormat = sampleFormatComboBox->currentIndex();
@@ -536,7 +570,9 @@ void ConfigDialog::browseSdkDir()
 
 void ConfigDialog::selectAudioInput()
 {
-	QList<QPair<QString, QString> > deviceList = m_configlists->getAudioInputDevices(RtModuleComboBox->currentText());
+	QList<QPair<QString, QString> > deviceList
+			= m_configlists->getAudioInputDevices(
+				RtModuleComboBox->itemData(RtModuleComboBox->currentIndex()).toString());
 	QMenu menu(this);
 	QVector<QAction*> actions;
 
@@ -561,7 +597,9 @@ void ConfigDialog::selectAudioInput()
 
 void ConfigDialog::selectAudioOutput()
 {
-	QList<QPair<QString, QString> > deviceList = m_configlists->getAudioOutputDevices(RtModuleComboBox->currentText());
+	QList<QPair<QString, QString> > deviceList
+			= m_configlists->getAudioOutputDevices(
+				RtModuleComboBox->itemData(RtModuleComboBox->currentIndex()).toString());
 	QMenu menu(this);
 	QVector<QAction*> actions;
 
