@@ -34,11 +34,14 @@
 #include "csound_standard_types.h"
 #endif
 
+
 #include "csoundengine.h"
 #include "widgetlayout.h"
 #include "console.h"
 #include "qutescope.h"  // Needed for passing the ud to the scope for display data
 #include "qutegraph.h"  // Needed for passing the ud to the graph for display data
+#include "midihandler.h"
+
 
 CsoundEngine::CsoundEngine(ConfigLists *configlists) :
 	m_options(configlists)
@@ -46,11 +49,13 @@ CsoundEngine::CsoundEngine(ConfigLists *configlists) :
 	QMutexLocker locker(&m_playMutex);
 	ud = new CsoundUserData();
 	ud->csEngine = this;
-	ud->csound = 0;
-	ud->perfThread = 0;
+	ud->csound = NULL;
+	ud->perfThread = NULL;
 	ud->flags = QCS_NO_FLAGS;
 	ud->mouseValues.resize(6); // For _MouseX _MouseY _MouseRelX _MouseRelY _MouseBut1 and _MouseBut2 channels
 	ud->wl = NULL;
+	ud->midiBuffer = NULL;
+	ud->playMutex = &m_playMutex;
 #ifdef QCS_PYTHONQT
 	ud->m_pythonCallback = "";
 #endif
@@ -62,9 +67,14 @@ CsoundEngine::CsoundEngine(ConfigLists *configlists) :
 
 #ifndef QCS_DESTROY_CSOUND
 	// Create only once
-	ud->csound=csoundCreate(0);
-	ud->perfThread = new CsoundPerformanceThread(ud->csound);
-	ud->perfThread->SetProcessCallback(CsoundEngine::csThread, (void*)ud);
+	ud->csound=csoundCreate( (void *) ud);
+//	ud->perfThread = new CsoundPerformanceThread(ud->csound);
+//	ud->perfThread->SetProcessCallback(CsoundEngine::csThread, (void*)ud);
+#ifdef CSOUND6
+//    csoundSetHostImplementedMIDIIO(ud->csound, 1);
+    ud->midiBuffer = csoundCreateCircularBuffer(ud->csound, 1024, sizeof(unsigned char));
+    Q_ASSERT(ud->midiBuffer);
+#endif
 #endif
 
 	eventQueue.resize(QCS_MAX_EVENTS);
@@ -76,7 +86,9 @@ CsoundEngine::CsoundEngine(ConfigLists *configlists) :
 
 	ud->runDispatcher = true;
 	m_msgUpdateThread = QtConcurrent::run(messageListDispatcher, (void *) ud);
-
+#ifdef QCS_DEBUGGER
+	m_debugging = false;
+#endif
 }
 
 CsoundEngine::~CsoundEngine()
@@ -88,6 +100,7 @@ CsoundEngine::~CsoundEngine()
 	m_msgUpdateThread.waitForFinished(); // Join the message thread
 	stop();
 #ifndef QCS_DESTROY_CSOUND
+	csoundDestroyCircularBuffer(ud->csound, ud->midiBuffer);
 	csoundDestroy(ud->csound);
 #endif
 	delete ud;
@@ -199,8 +212,8 @@ void CsoundEngine::outputValueCallback (CSOUND *csound,
 	// Called by the csound running engine when 'outvalue' opcode is used
 	// To pass data from Csound to CsoundQt
 	CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
-	if (channelType == &CS_VAR_TYPE_S) {
-		ud->csEngine->passOutString(channelName, (const char *) channelValuePtr);
+    if (channelType == &CS_VAR_TYPE_S) {
+        ud->csEngine->passOutString(channelName, (const char *) channelValuePtr);
 	}
 	else if (channelType == &CS_VAR_TYPE_K){
 		ud->csEngine->passOutValue(channelName, *((MYFLT *)channelValuePtr));
@@ -210,9 +223,9 @@ void CsoundEngine::outputValueCallback (CSOUND *csound,
 }
 
 void CsoundEngine::inputValueCallback (CSOUND *csound,
-								const char *channelName,
-								void *channelValuePtr,
-								const void *channelType)
+									   const char *channelName,
+									   void *channelValuePtr,
+									   const void *channelType)
 {
 	// Called by the csound running engine when 'invalue' opcode is used
 	// To pass data from qutecsound to Csound
@@ -221,7 +234,12 @@ void CsoundEngine::inputValueCallback (CSOUND *csound,
 		char *string = (char *) channelValuePtr;
 		QString newValue = ud->wl->getStringForChannel(channelName);
 		int maxlen = csoundGetChannelDatasize(csound, channelName);
-		strncpy(string, newValue.toLocal8Bit(), maxlen);
+		if (newValue.size() > 0) {
+			strncpy(string, newValue.toLocal8Bit(), maxlen -1 );
+			string[newValue.size()] = '\0';
+		} else {
+			string[0] = '\0';
+		}
 	}
 	else if (channelType == &CS_VAR_TYPE_K) {  // Not a string channel
 		//FIXME check if mouse tracking is active, and move this from here
@@ -253,6 +271,94 @@ void CsoundEngine::inputValueCallback (CSOUND *csound,
 	}
 }
 
+int CsoundEngine::midiInOpenCb(CSOUND *csound, void **ud, const char *devName)
+{
+	CsoundUserData *userData = (CsoundUserData *) csoundGetHostData(csound);
+	Q_UNUSED(devName);
+	Q_ASSERT(userData->midiBuffer);
+	if (userData) {
+		*ud = userData;
+	} else {
+		qDebug() << "Error! userData not set for midiInOpenCb";
+		return CSOUND_ERROR;
+	}
+	return CSOUND_SUCCESS;
+}
+
+int CsoundEngine::midiReadCb(CSOUND *csound, void *ud_, unsigned char *buf, int nBytes)
+{
+	CsoundUserData *ud = (CsoundUserData *) ud_;
+	Q_UNUSED(csound);
+	int count;
+	count = csoundReadCircularBuffer(ud->csound, ud->midiBuffer, buf, nBytes);
+	return count;
+}
+
+int CsoundEngine::midiInCloseCb(CSOUND *csound, void *ud)
+{
+    Q_UNUSED(csound);
+    Q_UNUSED(ud);
+	return CSOUND_SUCCESS;
+}
+
+int CsoundEngine::midiOutOpenCb(CSOUND *csound, void **ud, const char *devName)
+{
+	CsoundUserData *userData = (CsoundUserData *) csoundGetHostData(csound);
+	Q_UNUSED(devName);
+	Q_ASSERT(userData->midiBuffer);
+	if (userData) {
+		*ud = userData;
+	} else {
+		qDebug() << "Error! userData not set for midiInOpenCb";
+		return CSOUND_ERROR;
+	}
+	return CSOUND_SUCCESS;
+
+}
+
+int CsoundEngine::midiWriteCb(CSOUND *csound, void *ud_, const unsigned char *buf, int nBytes)
+{
+	CsoundUserData *userData = (CsoundUserData *) ud_;
+	std::vector<unsigned char> message;
+	Q_UNUSED(csound);
+	message.push_back(*buf++);
+	nBytes--;
+	while (nBytes--) {
+		if (*buf & 0x80) {
+			userData->midiHandler->sendMidiOut(&message);
+			message.clear();
+		}
+		message.push_back(*buf++);
+	}
+	if (message.size() > 0) {
+		userData->midiHandler->sendMidiOut(&message);
+	}
+	return 0;
+}
+
+int CsoundEngine::midiOutCloseCb(CSOUND *csound, void *ud)
+{
+	return 0;
+}
+
+const char *CsoundEngine::midiErrorStringCb(int)
+{
+
+	return NULL;
+}
+
+void CsoundEngine::queueMidiIn(std::vector< unsigned char > *message)
+{
+	if (ud->midiBuffer) {
+		csoundWriteCircularBuffer(ud->csound, ud->midiBuffer, message->data(), message->size()* sizeof(unsigned char));
+	}
+}
+
+void CsoundEngine::sendMidiOut(QVector<unsigned char> &message)
+{
+
+}
+
 #endif
 
 void CsoundEngine::makeGraphCallback(CSOUND *csound, WINDAT *windat, const char * /*name*/)
@@ -291,20 +397,21 @@ int CsoundEngine::keyEventCallback(void *userData,
 								   void *p,
 								   unsigned int type)
 {
-//	if (type != CSOUND_CALLBACK_KBD_EVENT)
-//		return 1;
+	if (type != CSOUND_CALLBACK_KBD_EVENT && type != CSOUND_CALLBACK_KBD_TEXT) {
+		return 1;
+	}
 	CsoundUserData *ud = (CsoundUserData *) userData;
 	//  WidgetLayout *wl = (WidgetLayout *) ud->wl;
 	int *value = (int *) p;
 	int key = ud->csEngine->popKeyPressEvent();
 	if (key >= 0) {
 		*value = key;
-		//    qDebug() << "Pressed: " << key;
+//			qDebug() << "Pressed: " << key;
 	}
-	else {
+	else if (type & CSOUND_CALLBACK_KBD_EVENT) {
 		key = ud->csEngine->popKeyReleaseEvent();
 		if (key >= 0) {
-			*value = key;
+			*value = key | 0x10000;
 			//       qDebug() << "Released: " << key;
 		}
 	}
@@ -366,60 +473,12 @@ void CsoundEngine::readWidgetValues(CsoundUserData *ud)
 		QHash<QString, QString>::const_iterator i;
 		QHash<QString, QString>::const_iterator end = ud->wl->newStringValues.constEnd();
 		for (i = ud->wl->newStringValues.constBegin(); i != end; ++i) {
-			if(csoundGetChannelPtr(ud->csound, &pvalue, i.key().toLocal8Bit().constData(),
-								   CSOUND_INPUT_CHANNEL | CSOUND_STRING_CHANNEL) == 0) {
-				char *string = (char *) pvalue;
-				strcpy(string, i.value().toLocal8Bit().constData());
-			}
+			csoundSetStringChannel(ud->csound, i.key().toLocal8Bit().data(),
+								   i.value().toLocal8Bit().data());
 		}
 		ud->wl->newStringValues.clear();
 		ud->wl->stringValueMutex.unlock();
 	}
-	//  for (int i = 0; i < ud->inputChannelNames.size(); i++) {
-	//    if(csoundGetChannelPtr(ud->csound, &pvalue, ud->inputChannelNames[i].toLocal8Bit().constData(),
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      double value = ud->wl->getValueForChannel(ud->inputChannelNames[i]);
-	//      *pvalue = (MYFLT) value;
-	////      *pvalue = (MYFLT) (ud->inputValues[i].toDouble());
-	//    }
-	//    else if(csoundGetChannelPtr(ud->csound, &pvalue, ud->inputChannelNames[i].toLocal8Bit().constData(),
-	//       CSOUND_INPUT_CHANNEL | CSOUND_STRING_CHANNEL) == 0) {
-	//      bool modified = false;
-	//      char *string = (char *) pvalue;
-	//      QString value = ud->wl->getStringForChannel(ud->inputChannelNames[i]);
-	//      if (modified) {
-	//        strcpy(string, value.toLocal8Bit().constData());
-	//      }
-	//    }
-	//    else {
-	//      qDebug() << "CsoundEngine::readWidgetValues invalid input channel: " << ud->inputChannelNames[i];
-	//    }
-	//  }
-	//FIXME check if mouse tracking is active
-	//  if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseX",
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      *pvalue = (MYFLT) ud->mouseValues[0];
-	//  }
-	//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseY",
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      *pvalue = (MYFLT) ud->mouseValues[1];
-	//  }
-	//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseRelX",
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      *pvalue = (MYFLT) ud->mouseValues[2];
-	//  }
-	//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseRelY",
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      *pvalue = (MYFLT) ud->mouseValues[3];
-	//  }
-	//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseBut1",
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      *pvalue = (MYFLT) ud->mouseValues[4];
-	//  }
-	//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseBut2",
-	//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-	//      *pvalue = (MYFLT) ud->mouseValues[5];
-	//  }
 }
 
 void CsoundEngine::writeWidgetValues(CsoundUserData *ud)
@@ -442,10 +501,12 @@ void CsoundEngine::writeWidgetValues(CsoundUserData *ud)
 				&& csoundGetChannelPtr(ud->csound, &pvalue,
 									   ud->outputStringChannelNames[i].toLocal8Bit().constData(),
 									   CSOUND_OUTPUT_CHANNEL | CSOUND_STRING_CHANNEL) == 0) {
-			QString value = QString((char *)pvalue);
-			if(ud->previousStringOutputValues[i] != value) {
-				ud->wl->setValue(ud->outputStringChannelNames[i],value);
-				ud->previousStringOutputValues[i] = value;
+            char chanString[128];
+            csoundGetStringChannel(ud->csound, ud->outputStringChannelNames[i].toLocal8Bit().constData(),
+                                                   chanString);
+            if(ud->previousStringOutputValues[i] != QString(chanString)) {
+                ud->wl->setValue(ud->outputStringChannelNames[i],QString(chanString));
+                ud->previousStringOutputValues[i] = QString(chanString);
 			}
 		}
 	}
@@ -469,16 +530,16 @@ void CsoundEngine::setWidgetLayout(WidgetLayout *wl)
 	connect(this, SIGNAL(passMessages(QString)), wl, SLOT(appendMessage(QString)));
 }
 
+void CsoundEngine::setMidiHandler(MidiHandler *mh)
+{
+	ud->midiHandler = mh;
+}
+
 void CsoundEngine::enableWidgets(bool enable)
 {
 	ud->enableWidgets = enable;
 }
 
-void CsoundEngine::setInitialDir(QString initialDir)
-{
-	m_initialDir = initialDir;
-}
-
 void CsoundEngine::registerConsole(ConsoleWidget *c)
 {
 	consoles.append(c);
@@ -506,7 +567,7 @@ void CsoundEngine::setConsoleBufferSize(int size)
 
 void CsoundEngine::keyPressForCsound(QString key)
 {
-	//  qDebug() << "CsoundEngine::keyPressForCsound " << key;
+//	  qDebug() << "CsoundEngine::keyPressForCsound " << key;
 	keyMutex.lock();
 	keyPressBuffer << key;
 	keyMutex.unlock();
@@ -514,7 +575,7 @@ void CsoundEngine::keyPressForCsound(QString key)
 
 void CsoundEngine::keyReleaseForCsound(QString key)
 {
-	//   qDebug() << "keyReleaseForCsound " << key;
+//	   qDebug() << "keyReleaseForCsound " << key;
 	keyMutex.lock();
 	keyReleaseBuffer << key;
 	keyMutex.unlock();
@@ -554,23 +615,25 @@ void CsoundEngine::evaluate(QString code)
 
 int CsoundEngine::popKeyPressEvent()
 {
-	keyMutex.lock();
 	int value = -1;
-	if (!keyPressBuffer.isEmpty()) {
-		value = (int) keyPressBuffer.takeFirst()[0].toLatin1();
+	if (keyMutex.tryLock()) {
+		if (!keyPressBuffer.isEmpty()) {
+			value = (int) keyPressBuffer.takeFirst()[0].toLatin1();
+		}
+		keyMutex.unlock();
 	}
-	keyMutex.unlock();
 	return value;
 }
 
 int CsoundEngine::popKeyReleaseEvent()
 {
-	keyMutex.lock();
 	int value = -1;
-	if (!keyReleaseBuffer.isEmpty()) {
-		value = (int) keyReleaseBuffer.takeFirst()[0].toLatin1() + 0x10000;
+	if (keyMutex.tryLock()) {
+		if (!keyReleaseBuffer.isEmpty()) {
+			value = (int) keyReleaseBuffer.takeFirst()[0].toLatin1();
+		}
+		keyMutex.unlock();
 	}
-	keyMutex.unlock();
 	return value;
 }
 
@@ -580,8 +643,9 @@ void CsoundEngine::processEventQueue()
 	eventMutex.lock();
 	while (eventQueueSize > 0) {
 		eventQueueSize--;
+		m_playMutex.lock();
 #ifdef QCS_DESTROY_CSOUND
-		if (ud->perfThread != 0) {
+		if (ud->perfThread) {
 			//ScoreEvent is not working
 			//      ud->perfThread->ScoreEvent(0, type, eventElements.size(), pFields);
 			//      qDebug() << "CsoundEngine::processEventQueue()" << eventQueue[eventQueueSize];
@@ -592,9 +656,12 @@ void CsoundEngine::processEventQueue()
 			qDebug() << "WARNING: ud->perfThread is NULL";
 		}
 #else
-		ud->perfThread
-				->InputMessage(eventQueue[eventQueueSize].toLatin1());
+		if (ud->perfThread) {
+			ud->perfThread
+					->InputMessage(eventQueue[eventQueueSize].toLatin1());
+		}
 #endif
+		m_playMutex.unlock();
 	}
 	eventMutex.unlock();
 }
@@ -613,14 +680,16 @@ void CsoundEngine::passOutString(QString channelName, QString value)
 int CsoundEngine::play(CsoundOptions *options)
 {
 	//  qDebug() << "CsoundEngine::play";
-	if (!isRunning()) {
-		m_options = *options;
-		return runCsound();
-	}
-	else {
+	QMutexLocker locker(&m_playMutex);
+	if (ud->perfThread && (ud->perfThread->GetStatus() == 0)) {
 		ud->perfThread->TogglePause();
 		return 0;
 	}
+	else {
+		m_options = *options;
+		locker.unlock();
+		return runCsound();
+	}
 }
 
 void CsoundEngine::stop()
@@ -631,7 +700,8 @@ void CsoundEngine::stop()
 
 void CsoundEngine::pause()
 {
-	if (isRunning() && ud->perfThread->GetStatus() == 0) {
+	QMutexLocker locker(&m_playMutex);
+	if (ud->perfThread && (ud->perfThread->GetStatus() == 0))  {
 		//ud->perfThread->Pause();
 		ud->perfThread->TogglePause();
 	}
@@ -704,6 +774,7 @@ int CsoundEngine::runCsound()
 #endif
 	eventQueueSize = 0; //Flush events gathered while idle
 	ud->audioOutputBuffer.allZero();
+	ud->msgRefreshTime = m_refreshTime*1000;
 
 	QDir::setCurrent(m_options.fileName1);
 	for (int i = 0; i < consoles.size(); i++) {
@@ -711,18 +782,47 @@ int CsoundEngine::runCsound()
 	}
 
 #ifdef QCS_DESTROY_CSOUND
-	ud->csound=csoundCreate(0);
+	ud->csound=csoundCreate((void *) ud);
+#ifdef CSOUND6
+	ud->midiBuffer = csoundCreateCircularBuffer(ud->csound, 1024, sizeof(unsigned char));
+	Q_ASSERT(ud->midiBuffer);
+#endif
 #endif
 
-	setupEnvironment(); // Environment variables must be set before csoundCompile
+#ifdef QCS_DEBUGGER
+	if(m_debugging) {
+		csoundDebuggerInit(ud->csound);
+		foreach(QVariantList bp, m_startBreakpoints) {
+			Q_ASSERT(bp.size() == 3);
+			if (bp[0].toString() == "instr") {
+				csoundSetInstrumentBreakpoint(ud->csound, bp[1].toDouble(), bp[2].toInt());
+			} else if (bp[0].toString() == "line") {
+//				csoundSetBreakpoint(ud->csound, bp[1].toInt(), bp[2].toInt());
+			} else {
+				qDebug() << "CsoundEngine::setStartingBreakpoints wrong breakpoint format";
+			}
+		}
+		csoundSetBreakpointCallback(ud->csound, &CsoundEngine::breakpointCallback, (void *) this);
+	}
+#endif
+	if(!m_options.useCsoundMidi) {
+#ifdef CSOUND6
+		csoundSetHostImplementedMIDIIO(ud->csound, 1);
+#endif
+		csoundSetExternalMidiInOpenCallback(ud->csound, &midiInOpenCb);
+		csoundSetExternalMidiReadCallback(ud->csound, &midiReadCb);
+		csoundSetExternalMidiInCloseCallback(ud->csound, &midiInCloseCb);
+		csoundSetExternalMidiOutOpenCallback(ud->csound, &midiOutOpenCb);
+		csoundSetExternalMidiWriteCallback(ud->csound, &midiWriteCb);
+		csoundSetExternalMidiOutCloseCallback(ud->csound, &midiOutCloseCb);
+		csoundSetExternalMidiErrorStringCallback(ud->csound, &midiErrorStringCb);
+    }
+
 
-	ud->msgRefreshTime = m_refreshTime*1000;
 #ifdef CSOUND6
 	csoundCreateMessageBuffer(ud->csound, 0);
-#endif
-	csoundSetHostData(ud->csound, (void *) ud);
-
-#ifndef CSOUND6
+	csoundFlushCircularBuffer(ud->csound, ud->midiBuffer);
+#else
 	// Message Callbacks must be set before compile, otherwise some information is missed
 	csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackThread);
 	csoundPreCompile(ud->csound);  //Need to run PreCompile to create the FLTK_Flags global variable
@@ -760,18 +860,26 @@ int CsoundEngine::runCsound()
 #ifdef CSOUND6
 	csoundRegisterKeyboardCallback(ud->csound,
 								   &CsoundEngine::keyEventCallback,
-								   (void *) ud, CSOUND_CALLBACK_KBD_EVENT | CSOUND_CALLBACK_KBD_TEXT);
+                                   (void *) ud, CSOUND_CALLBACK_KBD_EVENT | CSOUND_CALLBACK_KBD_TEXT);
 #else
 	csoundSetCallback(ud->csound,
 					  &CsoundEngine::keyEventCallback,
-					  (void *) ud, CSOUND_CALLBACK_KBD_EVENT | CSOUND_CALLBACK_KBD_TEXT);
+                      (void *) ud, CSOUND_CALLBACK_KBD_EVENT | CSOUND_CALLBACK_KBD_TEXT);
+#endif
+
+#ifdef QCS_RTMIDI
+    if (!m_options.useCsoundMidi) {
+        csoundSetOption(ud->csound, "-+rtmidi=hostbased");
+        csoundSetOption(ud->csound, "-M0");
+        csoundSetOption(ud->csound, "-Q0");
+    }
+#endif
+
 	csoundSetIsGraphable(ud->csound, true);
 	csoundSetMakeGraphCallback(ud->csound, &CsoundEngine::makeGraphCallback);
 	csoundSetDrawGraphCallback(ud->csound, &CsoundEngine::drawGraphCallback);
 	csoundSetKillGraphCallback(ud->csound, &CsoundEngine::killGraphCallback);
 	csoundSetExitGraphCallback(ud->csound, &CsoundEngine::exitGraphCallback);
-#endif
-
 
 	char **argv;
 	argv = (char **) calloc(33, sizeof(char*));
@@ -789,14 +897,7 @@ int CsoundEngine::runCsound()
         stop();
 		emit (errorLines(getErrorLines()));
 		return -3;
-	}
-#ifdef CSOUND6
-	csoundSetIsGraphable(ud->csound, true);
-	csoundSetMakeGraphCallback(ud->csound, &CsoundEngine::makeGraphCallback);
-	csoundSetDrawGraphCallback(ud->csound, &CsoundEngine::drawGraphCallback);
-	csoundSetKillGraphCallback(ud->csound, &CsoundEngine::killGraphCallback);
-	csoundSetExitGraphCallback(ud->csound, &CsoundEngine::exitGraphCallback);
-#endif
+    }
 	ud->zerodBFS = csoundGet0dBFS(ud->csound);
 	ud->sampleRate = csoundGetSr(ud->csound);
 	ud->numChnls = csoundGetNchnls(ud->csound);
@@ -816,8 +917,8 @@ void CsoundEngine::stopCsound()
 {
 	//  qDebug() << "CsoundEngine::stopCsound()";
 	//    perfThread->ScoreEvent(0, 'e', 0, 0);
-    QMutexLocker locker(&m_playMutex);
-	if (ud->perfThread != 0) {
+	m_playMutex.lock();
+	if (ud->perfThread) {
 		CsoundPerformanceThread *pt = ud->perfThread;
 		ud->perfThread = NULL;
 		pt->Stop();
@@ -826,7 +927,13 @@ void CsoundEngine::stopCsound()
 		pt->Join();
 		qDebug() << "CsoundEngine::stopCsound() joined";
 		delete pt;
+		m_playMutex.unlock();
 		cleanupCsound();
+#ifdef QCS_DEBUGGER
+		stopDebug();
+#endif
+	} else {
+		m_playMutex.unlock();
 	}
 #ifdef MACOSX_PRE_SNOW
 	// Put menu bar back
@@ -850,6 +957,11 @@ void CsoundEngine::cleanupCsound()
 #else
 		csoundRemoveCallback(ud->csound,
 							 &CsoundEngine::keyEventCallback);
+#endif
+#ifdef QCS_DEBUGGER
+		if(m_debugging) {
+			csoundDebuggerClean(ud->csound);
+		}
 #endif
 		csoundCleanup(ud->csound);
 		flushQueues();
@@ -859,11 +971,13 @@ void CsoundEngine::cleanupCsound()
 		csoundDestroyMessageBuffer(ud->csound);
 #endif
 #ifdef QCS_DESTROY_CSOUND
+
+		csoundDestroyCircularBuffer(ud->csound, ud->midiBuffer);
 		csoundDestroy(ud->csound);
+		ud->csound = NULL;
 #else
 		csoundReset(ud->csound);
 #endif
-		ud->csound = NULL;
 	}
 }
 
@@ -893,27 +1007,28 @@ void CsoundEngine::setupChannels()
 	controlChannelInfo_t *entry = channelList;
 #endif
 	MYFLT *pvalue;
+	QVector<QuteWidget *> widgets = ud->wl->getWidgets();
+	// Set channels values for existing channels (i.e. those declared with chn_*
+	// in the csound header
 	for (int i = 0; i < numChannels; i++) {
 		int chanType = csoundGetChannelPtr(ud->csound, &pvalue, entry->name,
 										   0);
-		QVector<QuteWidget *> widgets = ud->wl->getWidgets();
 		if (chanType & CSOUND_INPUT_CHANNEL) {
-//			ud->inputChannelNames << QString(entry->name);
-			if (chanType & CSOUND_CONTROL_CHANNEL) {
+			if ((chanType & CSOUND_CHANNEL_TYPE_MASK) == CSOUND_CONTROL_CHANNEL) {
 				ud->wl->valueMutex.lock();
 				foreach (QuteWidget *w, widgets) {
-					if (!w->getChannelName().isEmpty()) {
+					if (w->getChannelName() == QString(entry->name)) {
 						ud->wl->newValues.insert(w->getChannelName(), w->getValue());
 					}
-					if (!w->getChannel2Name().isEmpty()) {
+					if (w->getChannel2Name() == QString(entry->name)) {
 						ud->wl->newValues.insert(w->getChannel2Name(), w->getValue2());
 					}
 				}
 				ud->wl->valueMutex.unlock();
-			} else if (chanType & CSOUND_STRING_CHANNEL) {
+			} else if ((chanType & CSOUND_CHANNEL_TYPE_MASK) ==  CSOUND_STRING_CHANNEL) {
 				ud->wl->stringValueMutex.lock();
 				foreach (QuteWidget *w, widgets) {
-					if (!w->getChannelName().isEmpty()) {
+					if (w->getChannelName() == QString(entry->name)) {
 						ud->wl->newStringValues.insert(w->getChannelName(), w->getStringValue());
 					}
 				}
@@ -921,7 +1036,7 @@ void CsoundEngine::setupChannels()
 			}
 		}
 		if (chanType & CSOUND_OUTPUT_CHANNEL) { // Channels can be input and output at the same time
-			if (chanType & CSOUND_CONTROL_CHANNEL) {
+			if ((chanType & CSOUND_CHANNEL_TYPE_MASK) == CSOUND_CONTROL_CHANNEL) {
 				ud->outputChannelNames << QString(entry->name);
 				ud->previousOutputValues << 0;
 				foreach (QuteWidget *w, widgets) {
@@ -934,7 +1049,7 @@ void CsoundEngine::setupChannels()
 						continue;
 					}
 				}
-			} else if (chanType & CSOUND_STRING_CHANNEL) {
+			} else if ((chanType & CSOUND_CHANNEL_TYPE_MASK) == CSOUND_STRING_CHANNEL) {
 				ud->outputStringChannelNames << QString(entry->name);
 				ud->previousStringOutputValues << "";
 				foreach (QuteWidget *w, widgets) {
@@ -948,73 +1063,15 @@ void CsoundEngine::setupChannels()
 		entry++;
 	}
 	csoundDeleteChannelList(ud->csound, channelList);
-}
 
-void CsoundEngine::setupEnvironment()
-{
-	if (m_options.sadirActive){
-		int ret = csoundSetGlobalEnv("SADIR", m_options.sadir.toLocal8Bit().constData());
-		if (ret != 0) {
-			qDebug() << "CsoundEngine::runCsound() Error setting SADIR";
+	// Force creation of string channels for _Browse widgets
+	foreach (QuteWidget *w, widgets) {
+		if (w->getChannelName().startsWith("_Browse")) {
+			csoundGetChannelPtr(ud->csound, &pvalue, w->getChannelName().toLocal8Bit(),
+								CSOUND_INPUT_CHANNEL | CSOUND_OUTPUT_CHANNEL | CSOUND_STRING_CHANNEL);
+			ud->wl->newStringValues.insert(w->getChannelName(), w->getStringValue());
 		}
 	}
-	else {
-		csoundSetGlobalEnv("SADIR", "");
-	}
-	if (m_options.ssdirActive){
-		int ret = csoundSetGlobalEnv("SSDIR", m_options.ssdir.toLocal8Bit().constData());
-		if (ret != 0) {
-			qDebug() << "CsoundEngine::runCsound() Error setting SSDIR";
-		}
-	}
-	else {
-		csoundSetGlobalEnv("SSDIR", "");
-	}
-	if (m_options.sfdirActive){
-		int ret = csoundSetGlobalEnv("SFDIR", m_options.sfdir.toLocal8Bit().constData());
-		if (ret != 0) {
-			qDebug() << "CsoundEngine::runCsound() Error setting SFDIR";
-		}
-	}
-	else {
-		csoundSetGlobalEnv("SFDIR", "");
-	}
-	if (m_options.incdirActive){
-		int ret = csoundSetGlobalEnv("INCDIR", m_options.incdir.toLocal8Bit().constData());
-		if (ret != 0) {
-			qDebug() << "CsoundEngine::runCsound() Error setting INCDIR";
-		}
-	}
-	else {
-		csoundSetGlobalEnv("INCDIR", "");
-	}
-	// csoundGetEnv must be called after Compile or Precompile,
-	// But I need to set OPCODEDIR before compile.... So I can't know keep the old OPCODEDIR
-	if (m_options.opcodedirActive) {
-		csoundSetGlobalEnv("OPCODEDIR", m_options.opcodedir.toLatin1().constData());
-	}
-	if (m_options.opcodedir64Active) {
-		csoundSetGlobalEnv("OPCODEDIR64", m_options.opcodedir64.toLatin1().constData());
-	}
-	if (m_options.opcode6dir64Active) {
-		csoundSetGlobalEnv("OPCODE6DIR64", m_options.opcode6dir64.toLatin1().constData());
-	}
-#ifdef Q_OS_MAC
-	else { // Set opcode dir for bundled Csound if present
-#ifdef USE_DOUBLE
-		QString opcodedir = m_initialDir + "/CsoundQt.app/Contents/Frameworks/CsoundLib64.framework/Resources/Opcodes";
-#else
-		QString opcodedir = m_initialDir + "/CsoundQt.app/Contents/Frameworks/CsoundLib.framework/Resources/Opcodes";
-#endif
-		if (QFile::exists(opcodedir)) {
-#ifdef USE_DOUBLE
-			csoundSetGlobalEnv("OPCODEDIR64", opcodedir.toLocal8Bit().constData());
-#else
-			csoundSetGlobalEnv("OPCODEDIR", opcodedir.toLocal8Bit().constData());
-#endif
-		}
-	}
-#endif
 }
 
 void CsoundEngine::messageListDispatcher(void *data)
@@ -1023,10 +1080,12 @@ void CsoundEngine::messageListDispatcher(void *data)
 	CsoundUserData *ud_local = (CsoundUserData *) data;
 
 	while (ud_local->runDispatcher) {
-		if (ud_local->perfThread) {
-			if (ud_local->perfThread->GetStatus() != 0) { // In case score has ended
-				ud_local->csEngine->stop();
-			}
+		ud_local->playMutex->lock();
+		if (ud_local->perfThread && (ud_local->perfThread->GetStatus() != 0)) { // In case score has ended
+			ud_local->playMutex->unlock();
+			ud_local->csEngine->stop();
+		} else {
+			ud_local->playMutex->unlock();
 		}
 		CSOUND *csound = ud_local->csEngine->getCsound();
 
@@ -1144,5 +1203,129 @@ void CsoundEngine::setPythonConsole(PythonConsole *pc)
 {
 	ud->m_pythonConsole = pc;
 }
+#endif
+
+#ifdef QCS_DEBUGGER
+
+void CsoundEngine::breakpointCallback(CSOUND *csound, debug_bkpt_info_t *bkpt_info, void *udata)
+{
+    debug_instr_t *debug_instr = bkpt_info->breakpointInstr;
+	CsoundEngine *cs = (CsoundEngine *) udata;
+	// Copy variable list
+    debug_variable_t* vp = bkpt_info->instrVarList;
+	cs->variableMutex.lock();
+	cs->m_varList.clear();
+	while (vp) {
+        if (vp->name[0] != '#') {
+			QVariantList varDetails;
+            varDetails << vp->name;
+            if (strcmp(vp->typeName, "i") == 0
+                    || strcmp(vp->typeName, "k") == 0) {
+                varDetails << *((MYFLT *) vp->data);
+            } else if(strcmp(vp->typeName, "S") == 0) {
+                varDetails << (char *) vp->data;
+            } else if (strcmp(vp->typeName, "a") == 0) {
+                MYFLT *data = (MYFLT *) vp->data;
+                varDetails << *data << *(data + 1)
+                           << *(data + 2)<< *(data + 3);
+            } else {
+				varDetails << QVariant();
+			}
+            varDetails << vp->typeName;
+			cs->m_varList << varDetails;
+		}
+		vp = vp->next;
+	}
+	cs->variableMutex.unlock();
+    csoundDebugFreeVariables(csound, vp);
+    csoundDebugFreeInstrInstances(csound, debug_instr);
+
+    debug_instr = csoundDebugGetInstrInstances(csound);
+	//Copy active instrument list
+	cs->instrumentMutex.lock();
+    cs->m_instrumentList.clear();
+    while (debug_instr) {
+		QVariantList instance;
+        instance << debug_instr->p1;
+        instance << QString("%1 %2").arg(debug_instr->p2).arg(debug_instr->p3);
+        instance << debug_instr->kcounter;
+		cs->m_instrumentList << instance;
+        debug_instr = debug_instr->next;
+	}
+    cs->instrumentMutex.unlock();
+    csoundDebugFreeInstrInstances(csound, debug_instr);
+	emit cs->breakpointReached();
+}
+
+void CsoundEngine::setDebug()
+{
+    if (isRunning()) {
+        ud->perfThread->Pause();
+        csoundDebuggerInit(ud->csound);
+        csoundSetBreakpointCallback(ud->csound, &CsoundEngine::breakpointCallback, (void *) this);
+//        ud->perfThread->Play();
+    }
+    m_debugging = true;
+}
+
+void CsoundEngine::pauseDebug()
+{
+	if (isRunning() && m_debugging) {
+		csoundDebugStop(ud->csound);
+	}
+}
+
+void CsoundEngine::continueDebug()
+{
+	if (isRunning() && m_debugging) {
+		qDebug() << "CsoundEngine::continueDebug";
+		csoundDebugContinue(ud->csound);
+	}
+}
+
+void CsoundEngine::stopDebug()
+{
+	if (isRunning() && m_debugging) {
+		stop();
+	}
+	m_debugging = false;
+}
+
+void CsoundEngine::addInstrumentBreakpoint(double instr, int skip)
+{
+	if (isRunning() && m_debugging) {
+		csoundSetInstrumentBreakpoint(ud->csound, instr, skip);
+	}
+}
+
+void CsoundEngine::removeInstrumentBreakpoint(double instr)
+{
+	if (isRunning() && m_debugging) {
+		csoundRemoveInstrumentBreakpoint(ud->csound, instr);
+	}
+}
+
+void CsoundEngine::setStartingBreakpoints(QVector<QVariantList> bps)
+{
+	m_startBreakpoints = bps;
+}
+
+QVector<QVariantList> CsoundEngine::getVaribleList()
+{
+	QVector<QVariantList> outList;
+	variableMutex.lock();
+	outList = m_varList;
+	variableMutex.unlock();
+	return outList;
+}
+
+QVector<QVariantList> CsoundEngine::getInstrumentList()
+{
+	QVector<QVariantList> outList;
+	instrumentMutex.lock();
+	outList = m_instrumentList;
+	instrumentMutex.unlock();
+	return outList;
+}
 
 #endif
