@@ -23,6 +23,7 @@
 #include <cwindow.h>
 #include <csound.hpp>  // TODO These two are necessary for the WINDAT struct. Can they be moved?
 
+
 #include "documentpage.h"
 #include "documentview.h"
 #include "csoundengine.h"
@@ -36,17 +37,24 @@
 #include "highlighter.h"
 #include "widgetlayout.h"
 #include "console.h"
+#include "midilearndialog.h"
 
 #include "curve.h"
 #include "qutebutton.h"
 
 
 // TODO is is possible to move the editor to a separate child class, to be able to use a cleaner class?
-DocumentPage::DocumentPage(QWidget *parent, OpEntryParser *opcodeTree, ConfigLists *configlists):
+DocumentPage::DocumentPage(QWidget *parent, OpEntryParser *opcodeTree, ConfigLists *configlists, MidiLearnDialog *midiLearn):
 	BaseDocument(parent, opcodeTree, configlists)
 {
 	init(parent, opcodeTree);
 	m_view->showLineArea(true);
+	m_midiLearn = midiLearn;
+	foreach(WidgetLayout* wl, m_widgetLayouts) {
+		connect(wl, SIGNAL(changed()), this, SLOT(setModified()));
+		connect(wl, SIGNAL(widgetSelectedSignal(QuteWidget*)), this, SLOT(passSelectedWidget(QuteWidget*)));
+		connect(wl, SIGNAL(widgetUnselectedSignal(QuteWidget*)), this, SLOT(passUnselectedWidget(QuteWidget*)));
+	}
 }
 
 DocumentPage::~DocumentPage()
@@ -73,6 +81,16 @@ void DocumentPage::toggleLineArea()
 	m_view->toggleLineArea();
 }
 
+void DocumentPage::toggleParameterMode()
+{
+	m_view->toggleParameterMode();
+}
+
+//void DocumentPage::showParametersInEditor()
+//{
+//	m_view->parameterShowShortcutPressed();
+//}
+
 int DocumentPage::setTextString(QString &text)
 {
 	int ret = 0;
@@ -859,6 +877,11 @@ void DocumentPage::setTabStopWidth(int tabWidth)
 	m_view->setTabStopWidth(tabWidth);
 }
 
+void DocumentPage::setTabIndents(bool indents)
+{
+	m_view->setTabIndents(indents);
+}
+
 void DocumentPage::setLineWrapMode(QTextEdit::LineWrapMode wrapLines)
 {
 	m_view->setLineWrapMode(wrapLines);
@@ -874,6 +897,11 @@ void DocumentPage::setAutoComplete(bool autoComplete)
 	m_view->setAutoComplete(autoComplete);
 }
 
+void DocumentPage::setAutoParameterMode(bool autoParameterMode)
+{
+	m_view->setAutoParameterMode(autoParameterMode);
+}
+
 QString DocumentPage::getActiveSection()
 {
 	return m_view->getActiveSection();
@@ -972,11 +1000,6 @@ void DocumentPage::setConsoleColors(QColor fontColor, QColor bgColor)
 	m_console->setColors(fontColor, bgColor);
 }
 
-void DocumentPage::setInitialDir(QString initialDir)
-{
-	m_csEngine->setInitialDir(initialDir);
-}
-
 //DocumentView * DocumentPage::view()
 //{
 //  return m_view;
@@ -1127,6 +1150,22 @@ void DocumentPage::registerButton(QuteButton *b)
 	connect(b, SIGNAL(stop()), static_cast<CsoundQt *>(parent()), SLOT(stop()));
 }
 
+void DocumentPage::queueMidiIn(std::vector< unsigned char > *message)
+{
+	WidgetLayout *d = m_widgetLayouts[0];
+	unsigned int nBytes = message->size();
+	Q_ASSERT(nBytes < 4);
+	if ( ((d->midiWriteCounter + 1) % QCS_MAX_MIDI_QUEUE) != d->midiReadCounter) {
+		int index = d->midiWriteCounter;
+		d->midiWriteCounter++;
+		d->midiWriteCounter = d->midiWriteCounter % QCS_MAX_MIDI_QUEUE;
+		for (unsigned int i = 0; i < nBytes; i++) {
+			d->midiQueue[index][i] = (int)message->at(i);
+		}
+	}
+	m_csEngine->queueMidiIn(message);
+}
+
 void DocumentPage::init(QWidget *parent, OpEntryParser *opcodeTree)
 {
 	fileName = "";
@@ -1140,6 +1179,7 @@ void DocumentPage::init(QWidget *parent, OpEntryParser *opcodeTree)
 	m_view = new DocumentView(parent, opcodeTree);
 	connect(m_view, SIGNAL(evaluate(QString)), this, SLOT(evaluate(QString)));
 	connect(m_view,SIGNAL(setHelp()), this, SLOT(setHelp()));
+	connect(m_view, SIGNAL(closeExtraPanels()), this, SLOT(closeExtraPanels()));
 
 	// For logging of Csound output to file
 	connect(m_console, SIGNAL(logMessage(QString)),
@@ -1163,7 +1203,6 @@ void DocumentPage::init(QWidget *parent, OpEntryParser *opcodeTree)
 
 	// Connect for clearing marked lines and letting inspector know text has changed
 	connect(m_view, SIGNAL(contentsChanged()), this, SLOT(textChanged()));
-	connect(m_view, SIGNAL(opcodeSyntaxSignal(QString)), this, SLOT(opcodeSyntax(QString)));
 	//   connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(moved()));
 
 	connect(m_csEngine, SIGNAL(errorLines(QList<QPair<int, QString> >)),
@@ -1187,6 +1226,8 @@ WidgetLayout* DocumentPage::newWidgetLayout()
 {
 	WidgetLayout* wl = BaseDocument::newWidgetLayout();
 	connect(wl, SIGNAL(changed()), this, SLOT(setModified()));
+	connect(wl, SIGNAL(widgetSelectedSignal(QuteWidget*)), this, SLOT(passSelectedWidget(QuteWidget*)));
+	connect(wl, SIGNAL(widgetUnselectedSignal(QuteWidget*)), this, SLOT(passUnselectedWidget(QuteWidget*)));
 	//  connect(wl, SIGNAL(setWidgetClipboardSignal(QString)),
 	//        this, SLOT(setWidgetClipboard(QString)));
 	return wl;
@@ -1268,6 +1309,11 @@ int DocumentPage::runPython()
 	return p.exitCode();
 }
 
+void DocumentPage::closeExtraPanels()
+{
+	emit closeExtraPanelsSignal();
+}
+
 void DocumentPage::showWidgets(bool show)
 {
 	//  qDebug() << "DocumentPage::showWidgets" << show;
@@ -1290,6 +1336,17 @@ void DocumentPage::hideWidgets()
 	}
 }
 
+void DocumentPage::passSelectedWidget(QuteWidget *widget)
+{
+	m_midiLearn->setCurrentWidget(widget);
+}
+
+void DocumentPage::passUnselectedWidget(QuteWidget *widget)
+{
+	// TODO: Better options for unselecting widgets.
+	m_midiLearn->setCurrentWidget(NULL);
+}
+
 void DocumentPage::applyMacOptions(QStringList options)
 {
 	int index = options.indexOf(QRegExp("WindowBounds: .*"));
@@ -1384,11 +1441,6 @@ void DocumentPage::killLine()
 	m_view->killLine();
 }
 
-void DocumentPage::autoComplete()
-{
-	m_view->autoComplete();
-}
-
 void DocumentPage::setViewMode(int mode)
 {
 	m_view->setViewMode(mode);
@@ -1549,11 +1601,6 @@ void DocumentPage::setPanelTempo(LiveEventFrame *panel, double tempo)
 	}
 }
 
-void DocumentPage::opcodeSyntax(QString message)
-{
-	emit opcodeSyntaxSignal(message);
-}
-
 void DocumentPage::evaluatePython(QString code)
 {
 	emit evaluatePythonSignal(code);
