@@ -43,7 +43,7 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	}
 	connect(m_orcEditor, SIGNAL(textChanged()), this, SLOT(textChanged()));
 	connect(m_orcEditor, SIGNAL(cursorPositionChanged()),
-			this, SLOT(syntaxCheck()));
+            this, SLOT(syntaxCheck()));
 	setFocusProxy(m_mainEditor);  // for comment action from main application
 	internalChange = false;
 
@@ -53,6 +53,26 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 			this, SLOT(textChanged()));
 	connect(m_mainEditor, SIGNAL(cursorPositionChanged()),
 			this, SLOT(syntaxCheck()));
+	connect(m_mainEditor, SIGNAL(cursorPositionChanged()),
+			this, SLOT(updateContext()));
+	connect(m_mainEditor, SIGNAL(escapePressed()),
+			this, SLOT(escapePressed()));
+	connect(m_mainEditor, SIGNAL(newLine()),
+			this, SLOT(indentNewLine()));
+	connect(m_mainEditor, SIGNAL(requestIndent()),
+			this, SLOT(indent()));
+	connect(m_mainEditor, SIGNAL(requestUnindent()),
+			this, SLOT(unindent()));
+	connect(m_mainEditor, SIGNAL(tabPressed()),
+			this, SLOT(nextParameter()));
+	connect(m_mainEditor, SIGNAL(backtabPressed()),
+			this, SLOT(prevParameter()));
+	connect(m_mainEditor, SIGNAL(arrowPressed()),
+			this, SLOT(exitParameterMode()));
+	connect(m_mainEditor, SIGNAL(enterPressed()),
+			this, SLOT(finishParameterMode()));
+    connect(m_mainEditor, SIGNAL(mouseReleased()),
+            this, SLOT(exitParameterMode()));
 
 	//TODO put this for line reporting for score editor
 	//  connect(scoreEditor, SIGNAL(textChanged()),
@@ -72,18 +92,28 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	syntaxMenu->setPalette(p);
 	connect(syntaxMenu,SIGNAL(keyPressed(QString)),
 			m_mainEditor, SLOT(insertPlainText(QString)));
-	//  connect(syntaxMenu,SIGNAL(aboutToHide()),
-	//          this, SLOT(destroySyntaxMenu()));
 
 	setViewMode(1);
 	setViewMode(0);  // To force a change
 	setAcceptDrops(true);
+
+	m_hoverWidget = new QWidget(m_mainEditor);
+	m_hoverText= new QLabel("Hello", m_hoverWidget);
+	m_hoverText->show();
+	m_hoverText->setStyleSheet(
+				"QLabel {"
+				"border: 1px solid black;"
+//				"border-radius: 4px;"
+				"padding: 2px;"
+				"background-color: #ffffcc;"
+				"};");
+	m_hoverText->setWordWrap(true);
+	m_hoverWidget->hide();
 }
 
 DocumentView::~DocumentView()
 {
 	disconnect(this, 0,0,0);
-	//  delete m_highlighter;
 }
 
 bool DocumentView::isModified()
@@ -108,6 +138,207 @@ void DocumentView::print(QPrinter *printer)
 	m_mainEditor->print(printer);
 }
 
+void DocumentView::updateContext()
+{
+	QStringList contextStart;
+	contextStart << "<CsInstruments>" << "<CsScore" << "<CsOptions>";
+	QTextCursor cursor = m_mainEditor->textCursor();
+	cursor.select(QTextCursor::LineUnderCursor);
+	QString line = cursor.selection().toPlainText();
+	int outerContext = 0;
+	while (!cursor.atStart()) {
+		foreach(QString startText, contextStart) {
+			if (line.contains(startText)) {
+				outerContext = contextStart.indexOf(startText) + 1;
+				break;
+			}
+		}
+		cursor.movePosition(QTextCursor::Up);
+		cursor.select(QTextCursor::BlockUnderCursor);
+		line = cursor.selection().toPlainText();
+		cursor.movePosition(QTextCursor::StartOfBlock);
+	}
+	if (outerContext == 1) { // Instrument section
+		QString endText = "</CsInstruments>";
+		QTextCursor linecursor = cursor;
+		linecursor.select(QTextCursor::LineUnderCursor);
+		line = linecursor.selection().toPlainText();
+		linecursor.movePosition(QTextCursor::EndOfLine);
+		while (!linecursor.atEnd() && !line.contains(endText)) {
+			cursor.movePosition(QTextCursor::NextBlock, QTextCursor::KeepAnchor);
+			cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
+			linecursor = cursor;
+			linecursor.select(QTextCursor::LineUnderCursor);
+			line = linecursor.selection().toPlainText();
+			linecursor.movePosition(QTextCursor::EndOfLine);
+		}
+		QString orc = cursor.selection().toPlainText();
+		updateOrcContext(orc);
+	}
+}
+
+void DocumentView::updateOrcContext(QString orc)
+{
+	// TODO: add string as context
+	QStringList innerContextStart;
+	innerContextStart << "instr" << "opcode";
+	QTextCursor cursor = m_mainEditor->textCursor();
+	cursor.select(QTextCursor::LineUnderCursor);
+	QString line = cursor.selection().toPlainText().trimmed();
+	int innerContext = 0;
+	while (!cursor.atStart()) {
+		foreach(QString startText, innerContextStart) {
+			if (line.startsWith(startText)) {
+				innerContext = innerContextStart.indexOf(startText) + 1;
+				break;
+			}
+		}
+		cursor.movePosition(QTextCursor::Up);
+		cursor.select(QTextCursor::BlockUnderCursor);
+		line = cursor.selection().toPlainText().trimmed();
+		cursor.movePosition(QTextCursor::StartOfBlock);
+	}
+	cursor.movePosition(QTextCursor::NextBlock);
+	QString endText = "endin";
+	QTextCursor linecursor = cursor;
+	linecursor.select(QTextCursor::LineUnderCursor);
+	line = linecursor.selection().toPlainText().trimmed();
+	linecursor.movePosition(QTextCursor::EndOfLine);
+	while (!linecursor.atEnd() && !line.startsWith(endText)) {
+		cursor.movePosition(QTextCursor::NextBlock, QTextCursor::KeepAnchor);
+		cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
+		linecursor = cursor;
+		linecursor.select(QTextCursor::LineUnderCursor);
+		line = linecursor.selection().toPlainText().trimmed();
+		linecursor.movePosition(QTextCursor::EndOfLine);
+	}
+	QString instr = cursor.selection().toPlainText();
+	QStringList lines = instr.split("\n", QString::SkipEmptyParts);
+
+	m_localVariables.clear();
+	foreach(QString line, lines) {
+		if (line.trimmed().startsWith(";")) {
+			continue;
+		}
+		QStringList words = line.split(QRegExp("[\\s,]"), QString::SkipEmptyParts);
+		int opcodeIndex = -1;
+		foreach(QString word, words) {
+			if (m_opcodeTree->isOpcode(word)) {
+				opcodeIndex = words.indexOf(word);
+			}
+		}
+		if(opcodeIndex > 0) {
+			words = words.mid(0, opcodeIndex);
+			foreach(QString word, words) {
+				if (word.at(0).isLetter()) {
+					m_localVariables << word.remove(QRegExp("[\\(\\)]"));
+				}
+			}
+		}
+	}
+	m_localVariables.removeDuplicates();
+	m_localVariables.sort();
+}
+
+void DocumentView::nextParameter()
+{
+	QChar character = m_mainEditor->document()->characterAt(m_mainEditor->textCursor().position());
+	// First move forward a word
+	while(!m_mainEditor->textCursor().atBlockEnd() &&
+		  !(character.isSpace() || character == ',' || character == '(' || character == ')'
+			|| character == '[' || character == ']' || character == '\\')) {
+		m_mainEditor->moveCursor(QTextCursor::NextCharacter);
+		character = m_mainEditor->document()->characterAt(m_mainEditor->textCursor().position());
+	}
+	// Then move forward all white space
+	while(!m_mainEditor->textCursor().atBlockEnd() &&
+		  (character.isSpace() || character == ',' || character == '(' || character == ')'
+		   || character == '[' || character == ']' || character == '\\')) {
+		m_mainEditor->moveCursor(QTextCursor::NextCharacter);
+		character = m_mainEditor->document()->characterAt(m_mainEditor->textCursor().position());
+	}
+	if (m_mainEditor->textCursor().atBlockStart()) {
+		m_mainEditor->moveCursor(QTextCursor::PreviousCharacter);
+	}
+	QTextCursor cursor = m_mainEditor->textCursor();
+	cursor.select(QTextCursor::WordUnderCursor);
+	if (cursor.selectedText() == "\"") {
+		cursor.movePosition(QTextCursor::NextCharacter);
+		while (!(cursor.selectedText().endsWith("\"") || cursor.atBlockEnd())) {
+			cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+		}
+		if (cursor.selectedText().endsWith("\"")) {
+			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor);
+		}
+	}
+	m_mainEditor->setTextCursor(cursor);
+	QRect cursorRect = m_mainEditor->cursorRect();
+	updateHoverText(cursorRect.x(), cursorRect.y(), cursor.selectedText());
+}
+
+void DocumentView::prevParameter()
+{
+	if (!m_mainEditor->textCursor().atBlockStart()) {
+		m_mainEditor->moveCursor(QTextCursor::PreviousCharacter);
+	}
+	QChar character = m_mainEditor->document()->characterAt(m_mainEditor->textCursor().position());
+	while(!m_mainEditor->textCursor().atBlockStart() &&
+		  !(character.isSpace() || character == ',' || character == '(' || character == ')'
+			|| character == '[' || character == ']' || character == '\\')) {
+		m_mainEditor->moveCursor(QTextCursor::PreviousCharacter);
+		character = m_mainEditor->document()->characterAt(m_mainEditor->textCursor().position());
+	}
+	while(!m_mainEditor->textCursor().atBlockStart() &&
+		  (character.isSpace() || character == ',' || character == '(' || character == ')'
+		   || character == '[' || character == ']' || character == '\\')) {
+		m_mainEditor->moveCursor(QTextCursor::PreviousCharacter);
+		character = m_mainEditor->document()->characterAt(m_mainEditor->textCursor().position());
+	}
+	QTextCursor cursor = m_mainEditor->textCursor();
+	cursor.select(QTextCursor::WordUnderCursor);
+	if (cursor.selectedText().startsWith("\"")) {
+		cursor.movePosition(QTextCursor::PreviousCharacter);
+		while (!(cursor.selectedText().startsWith("\"") || cursor.atBlockEnd())) {
+			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor);
+		}
+		if (cursor.selectedText().startsWith("\"")) {
+			cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+		}
+	}
+	m_mainEditor->setTextCursor(cursor);
+	QRect cursorRect = m_mainEditor->cursorRect();
+	updateHoverText(cursorRect.x(), cursorRect.y(), cursor.selectedText());
+}
+
+void DocumentView::updateHoverText(int x, int y, QString text)
+{
+	QString displayText = m_currentOpcodeText;
+	if (text.isEmpty()) {
+		return;
+	}
+	displayText.replace(text, "<b>" + text + "</b>");
+	m_hoverText->setText(displayText);
+	m_hoverText->adjustSize();
+	QRect textRect = m_hoverText->contentsRect();
+	int textWidth = textRect.width() + 10;
+	if (textRect.width() + 10 > this->width()) {
+		textRect.setWidth(this->width());
+		textWidth = this->width();
+	}
+	int xoffset = 0;
+	if (m_currentOpcodeText.contains(text)) {
+		xoffset = 10 + textRect.width() * (m_currentOpcodeText.indexOf(text)/(float) (m_currentOpcodeText.indexOf("<br />") + 5));
+	}
+	if (xoffset > x) {
+		xoffset = x;
+	} else if (textWidth + x - xoffset > this->width()) {
+		xoffset = - (this->width() - (textWidth + x - xoffset));
+	}
+	m_hoverWidget->setGeometry(x - xoffset, y-textRect.height() - 10, textWidth, textRect.height() + 10);
+
+	m_hoverWidget->show();
+}
+
 void DocumentView::setModified(bool mod)
 {
 	//  qDebug() << "DocumentView::setModified";
@@ -166,6 +397,11 @@ void DocumentView::setAutoComplete(bool autoComplete)
 	m_autoComplete = autoComplete;
 }
 
+void DocumentView::setAutoParameterMode(bool autoParameterMode)
+{
+	m_autoParameterMode = autoParameterMode;
+}
+
 void DocumentView::setViewMode(int mode)
 {
 	if (m_viewMode == mode)
@@ -303,29 +539,59 @@ QString DocumentView::getActiveSection()
 	// Will return all document if there are no ## boundaries (for any kind of file)
 	QString section;
 	if (m_viewMode < 2) {
-		QTextCursor cursor = m_mainEditor->textCursor();
-		cursor.select(QTextCursor::LineUnderCursor);
-		bool sectionStart = cursor.selectedText().simplified().startsWith("##");
-		while (!sectionStart && !cursor.anchor() == 0) {
-			cursor.movePosition(QTextCursor::PreviousBlock);
+		if (m_mode == EDIT_PYTHON_MODE) {
+			QTextCursor cursor = m_mainEditor->textCursor();
 			cursor.select(QTextCursor::LineUnderCursor);
-			sectionStart = cursor.selectedText().simplified().startsWith("##");
-		}
-		int start = cursor.anchor();
-		cursor = m_mainEditor->textCursor();
-		cursor.movePosition(QTextCursor::NextBlock);
-		cursor.select(QTextCursor::LineUnderCursor);
-		bool sectionEnd = cursor.selectedText().simplified().startsWith("##");
-		while (!sectionEnd && !cursor.atEnd()) {
+			bool sectionStart = cursor.selectedText().simplified().startsWith("##");
+			while (!sectionStart && !cursor.anchor() == 0) {
+				cursor.movePosition(QTextCursor::PreviousBlock);
+				cursor.select(QTextCursor::LineUnderCursor);
+				sectionStart = cursor.selectedText().simplified().startsWith("##");
+			}
+			int start = cursor.anchor();
+			cursor = m_mainEditor->textCursor();
 			cursor.movePosition(QTextCursor::NextBlock);
 			cursor.select(QTextCursor::LineUnderCursor);
-			sectionEnd = cursor.selectedText().simplified().startsWith("##");
+			bool sectionEnd = cursor.selectedText().simplified().startsWith("##");
+			while (!sectionEnd && !cursor.atEnd()) {
+				cursor.movePosition(QTextCursor::NextBlock);
+				cursor.select(QTextCursor::LineUnderCursor);
+				sectionEnd = cursor.selectedText().simplified().startsWith("##");
+			}
+			cursor.movePosition(QTextCursor::EndOfLine);
+			cursor.setPosition(start, QTextCursor::KeepAnchor);
+			m_mainEditor->setTextCursor(cursor);
+			section = cursor.selectedText();
+			section.replace(QChar(0x2029), QChar('\n'));
+		} else if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE) {
+			QTextCursor cursor = m_mainEditor->textCursor();
+			cursor.select(QTextCursor::LineUnderCursor);
+			QString text = cursor.selectedText().simplified();
+			bool sectionStart = text.startsWith("instr") || text.startsWith(";;");
+			while (!sectionStart && !cursor.anchor() == 0) {
+				cursor.movePosition(QTextCursor::PreviousBlock);
+				cursor.select(QTextCursor::LineUnderCursor);
+				text = cursor.selectedText().simplified();
+				sectionStart = text.startsWith("instr") || text.startsWith(";;");
+			}
+			int start = cursor.anchor();
+			cursor = m_mainEditor->textCursor();
+			cursor.movePosition(QTextCursor::NextBlock);
+			cursor.select(QTextCursor::LineUnderCursor);
+			text = cursor.selectedText().simplified();
+			bool sectionEnd = text.startsWith("endin") || text.startsWith(";;");
+			while (!sectionEnd && !cursor.atEnd()) {
+				cursor.movePosition(QTextCursor::NextBlock);
+				cursor.select(QTextCursor::LineUnderCursor);
+				text = cursor.selectedText().simplified();
+				sectionEnd = text.startsWith("endin") || text.startsWith(";;");
+			}
+			cursor.movePosition(QTextCursor::EndOfLine);
+			cursor.setPosition(start, QTextCursor::KeepAnchor);
+			m_mainEditor->setTextCursor(cursor);
+			section = cursor.selectedText();
+			section.replace(QChar(0x2029), QChar('\n'));
 		}
-		cursor.movePosition(QTextCursor::EndOfLine);
-		cursor.setPosition(start, QTextCursor::KeepAnchor);
-		m_mainEditor->setTextCursor(cursor);
-		section = cursor.selectedText();
-		section.replace(QChar(0x2029), QChar('\n'));
 	}
 	else { //  Split view
 		// TODO check properly for line number also from other editors
@@ -341,8 +607,8 @@ QString DocumentView::getActiveText()
 		QTextCursor cursor = m_mainEditor->textCursor();
 		selection = cursor.selectedText();
 		if (selection == "") {
-			cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::MoveAnchor);
-			cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
+			cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
+			cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
 			selection = cursor.selectedText();
 		}
 		selection.replace(QChar(0x2029), QChar('\n'));
@@ -357,44 +623,27 @@ QString DocumentView::getActiveText()
 	return selection;
 }
 
-
-//void DocumentView::updateDocumentModel()
-//{
-//  // this should update the document model when needed
-//  // e.g. on run or save or when widgets or presets have been modified in text format
-//  // maybe the document model pointer should be passed and processed here.
-//}
-//
-//void DocumentView::updateFromDocumentModel()
-//{
-//  // this should update from the document model when needed
-//  // e.g. on loading, widget changes from widget layout
-//}
-
 void DocumentView::syntaxCheck()
 {
 	// TODO implment for multiple views
 
-	int line = currentLine();
-	emit(lineNumberSignal(line));
-
 	TextEditor *editor;
 	if (m_viewMode < 2) {
+		// TODO rather than check this, store the current active one?
 		editor = m_mainEditor;
 	}
 	else { //  Split view
 		editor = (TextEditor *) sender();
 	}
 	QTextCursor cursor = editor->textCursor();
-	cursor.select(QTextCursor::LineUnderCursor);
+	cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
 	QStringList words = cursor.selectedText().split(QRegExp("\\b"));
-	foreach(QString word, words) {
-		// We need to remove all not possibly opcode
-		word.remove(QRegExp("[^\\d\\w]"));
-		if (!word.isEmpty()) {
+	for(int i = 0; i < words.size(); i++) {
+		QString word = words[words.size() - i - 1];
+		if (m_opcodeTree->isOpcode(word)) {
 			QString syntax = m_opcodeTree->getSyntax(word);
 			if(!syntax.isEmpty()) {
-				emit(opcodeSyntaxSignal(syntax));
+				m_currentOpcodeText = syntax;
 				return;
 			}
 		}
@@ -407,73 +656,143 @@ void DocumentView::textChanged()
 		internalChange = false;
 		return;
 	}
-	TextEditor *editor = (TextEditor *) sender();
+	TextEditor *editor = m_mainEditor;
 	unmarkErrorLines();
-	if (m_mode == 0 || m_mode == 3) {  // CSD or ORC mode
+
+//	parameterButton->setVisible(false);
+
+	if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE) {  // CSD or ORC mode
 		if (m_autoComplete) {
 			QTextCursor cursor = editor->textCursor();
 			int curIndex = cursor.position();
+//			if (cursor) {
+//				word.chop(1);
+//			}
 			cursor.select(QTextCursor::WordUnderCursor);
 			QString word = cursor.selectedText();
+			if (word == ",") {
+				cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::MoveAnchor, 2);
+				cursor.movePosition(QTextCursor::StartOfWord);
+				cursor.select(QTextCursor::WordUnderCursor);
+				word = cursor.selectedText();
+			}
 			QTextCursor lineCursor = editor->textCursor();
 			lineCursor.select(QTextCursor::LineUnderCursor);
 			QString line = lineCursor.selectedText();
+
 			int commentIndex = -1;
+			bool useFunction = false;
 			if (line.indexOf(";") != -1) {
 				commentIndex = lineCursor.position() - line.length() + line.indexOf(";");
 				if (commentIndex < curIndex)
 					return;
 			}
-			if (line.contains("opcode") || line.contains("instr")  || line.contains("=") || line.contains("\"")) { // Don't pop menu in these cases.
+			if (line.contains("opcode") || line.contains("instr")) { // Don't pop menu in these cases.
 				return;
 			}
-			if (line.indexOf(QRegExp("^\\s*[^kaigSf]\\w+\\s+\\w")) >= 0 || line.indexOf(QRegExp("\\s+\\w+\\s+\\w")) >= 0) {
-				return;
+			if (line.indexOf(QRegExp("\\s*\\w+\\s+\\w+\\s+")) >= 0) {
+				useFunction = true;
 			}
-			if (word.size() > 2 && !word.startsWith("\"")
-					&& cursor.position() > cursor.anchor() // Only at the end of the word
-					) {
-				QVector<Opcode> syntax = m_opcodeTree->getPossibleSyntax(word);
-				if (syntax.size() > 0) {
-					//          if (syntaxMenu == 0) {
-					//            createSyntaxMenu();
-					//          }
-					bool allEqual = true;
-					for(int i = 0; i < syntax.size(); i++) {
-						if (syntax[i].opcodeName != word) {
-							allEqual = false;
+			if (cursor.position() > cursor.anchor()) { // Only at the end of the word
+				if (word.size() > 0 && !word.startsWith("\"")) {
+					QStringList vars;
+					syntaxMenu->clear();
+					foreach(QString var, m_localVariables) {
+						if (var.endsWith(',')) {
+							var.chop(1);
+						}
+						if (var.startsWith(word) && word != var) {
+							vars << var;
 						}
 					}
-					if (!allEqual && syntax.size() > 0) {
-						syntaxMenu->clear();
-						for(int i = 0; i < syntax.size(); i++) {
-							QString text = syntax[i].opcodeName;
-							if (syntax[i].outArgs.simplified().startsWith("a")) {
-								text += " (audio-rate)";
-							}
-							else if (syntax[i].outArgs.simplified().startsWith("k")) {
-								text += " (control-rate)";
-							}
-							else if (syntax[i].outArgs.simplified().startsWith("x")) {
-								text += " (multi-rate)";
-							}
-							else if (syntax[i].outArgs.simplified().startsWith("S")) {
-								text += " (string output)";
-							}
-							else if (syntax[i].outArgs.simplified().startsWith("f")) {
-								text += " (pvs)";
-							}
-							QString syntaxText = syntax[i].outArgs.simplified();
-							if (!syntax[i].outArgs.isEmpty())
-								syntaxText += " ";
-							syntaxText += syntax[i].opcodeName.simplified();
-							if (!syntax[i].inArgs.isEmpty()) {
-								syntaxText += " " + syntax[i].inArgs.simplified();
-							}
-							QAction *a = syntaxMenu->addAction(text,
-															   this, SLOT(insertTextFromAction()));
-							a->setData(syntaxText);
+					foreach(QString var, vars) {
+						QAction *a = syntaxMenu->addAction(var,
+														   this, SLOT(insertParameterText()));
+						a->setData(var);
+						if(vars.indexOf(var) == 0) {
+							syntaxMenu->setDefaultAction(a);
 						}
+					}
+					if (word.size() > 2) {
+						// check for autcompletion from all words in text editor
+						QString wholeText = editor->toPlainText();
+						foreach (QString s, QStringList()<<"+" << "-" << "*" << "/" <<"="<<"#"<<"&"<<","
+								 << "\""<<"\'"<<"|" << "[" << "]" << "(" << ")" << "<" << ">"
+								 << "." << ";") // get rid of specific symbols
+							wholeText.replace(s," ");
+						QStringList allWords = wholeText.simplified().split(" ");
+						QStringList menuWords;
+						allWords.removeDuplicates();
+						allWords.replaceInStrings(QRegExp("^\\d*$"),""); // remove numbers - not good enough regexp, '.' stays
+						allWords.removeAll("");
+						foreach(QString theWord, allWords) {
+							if (theWord.toLower().startsWith(word.toLower()) && word != theWord) {
+								menuWords << theWord;
+							}
+						}
+						foreach(QString theWord, menuWords) {
+							QAction *a = syntaxMenu->addAction(theWord,
+															   this, SLOT(insertAutoCompleteText()));
+							a->setData(theWord);
+							if(menuWords.indexOf(theWord) == 0) {
+								syntaxMenu->setDefaultAction(a); // vaata, et allpool seda ei tühistaks
+							}
+						}
+						// and then for opcodes and parameters
+						QVector<Opcode> syntax = m_opcodeTree->getPossibleSyntax(word);
+						if (syntax.size() > 0) {
+							syntaxMenu->addSeparator();
+							bool allEqual = true;
+							for(int i = 0; i < syntax.size(); i++) {
+								if (syntax[i].opcodeName != word) {
+									allEqual = false;
+								}
+							}
+							if (!allEqual && syntax.size() > 0) {
+								for(int i = 0; i < syntax.size(); i++) {
+									QString text = syntax[i].opcodeName;
+									if (syntax[i].outArgs.simplified().startsWith("a")) {
+										text += " (audio-rate)";
+									}
+									else if (syntax[i].outArgs.simplified().startsWith("k")) {
+										text += " (control-rate)";
+									}
+									else if (syntax[i].outArgs.simplified().startsWith("x")) {
+										text += " (multi-rate)";
+									}
+									else if (syntax[i].outArgs.simplified().startsWith("S")) {
+										text += " (string output)";
+									}
+									else if (syntax[i].outArgs.simplified().startsWith("f")) {
+										text += " (pvs)";
+									}
+									QString syntaxText;
+									if (useFunction) {
+										syntaxText = syntax[i].opcodeName.simplified();
+										syntaxText += "(";
+										syntaxText += syntax[i].inArgs.simplified();
+										syntaxText += ")";
+									} else {
+										syntaxText= syntax[i].outArgs.simplified();
+										if (!syntax[i].outArgs.isEmpty())
+											syntaxText += " ";
+										syntaxText += syntax[i].opcodeName.simplified();
+										if (!syntax[i].inArgs.isEmpty()) {
+											syntaxText += " " + syntax[i].inArgs.simplified();
+										}
+									}
+									QAction *a = syntaxMenu->addAction(text,
+																	   this, SLOT(insertAutoCompleteText()));
+									a->setData(syntaxText);
+									a->setToolTip(syntaxText);
+									if (i == 0) {
+										syntaxMenu->setDefaultAction(a);
+									}
+								}
+							}
+						}
+					}
+					if (syntaxMenu->defaultAction() != NULL) {
 						QRect r =  editor->cursorRect();
 						QPoint p = QPoint(r.x() + r.width(), r.y() + r.height());
 						QPoint globalPoint =  editor->mapToGlobal(p);
@@ -481,20 +800,84 @@ void DocumentView::textChanged()
 						//syntaxMenu->popup(globalPoint);
 						syntaxMenu->move(globalPoint);
 						syntaxMenu->show();
-						editor->setFocus(Qt::OtherFocusReason);
-					}
-					else {
-						destroySyntaxMenu();
 					}
+					//						editor->setFocus(Qt::OtherFocusReason);
+				}
+				else {
+					destroySyntaxMenu();
 				}
 			}
 		}
 		syntaxCheck();
 	}
-	else if (m_mode == 1) { // Python Mode
+	else if (m_mode == EDIT_PYTHON_MODE) { // Python Mode
 		// Nothing for now
 	}
+}
 
+void DocumentView::escapePressed()
+{
+	// TODO implment for multiple views
+	if (m_viewMode < 2) {
+		if (m_mainEditor->getParameterMode()) {
+			// Force unselecting
+			m_mainEditor->moveCursor(QTextCursor::NextCharacter);
+			m_mainEditor->moveCursor(QTextCursor::PreviousCharacter);
+			exitParameterMode();
+		} else {
+			emit closeExtraPanels();
+		}
+	}
+	else {
+		qDebug() << "escapePressed() not implemented for split view";
+	}
+}
+
+void DocumentView::finishParameterMode()
+{
+	// TODO implment for multiple views
+	if (m_viewMode < 2) {
+		if (m_mainEditor->getParameterMode()) {
+			killToEnd();
+			exitParameterMode();
+		}
+	} else {
+		qDebug() << "finishParameterMode() not implemented for split view";
+	}
+}
+
+void DocumentView::exitParameterMode()
+{
+	// TODO implment for multiple views
+	if (m_viewMode < 2) {
+		m_mainEditor->setParameterMode(false);
+	} else {
+		qDebug() << "exitParameterMode() not implemented for split view";
+	}
+	m_hoverWidget->hide();
+//	parameterButton->setVisible(false);
+}
+
+void DocumentView::indentNewLine()
+{
+	QTextCursor linecursor = m_mainEditor->textCursor();
+	if (m_mode == EDIT_PYTHON_MODE){
+		linecursor.movePosition(QTextCursor::PreviousCharacter);
+		linecursor.select(QTextCursor::LineUnderCursor);
+		QString line = linecursor.selectedText();
+		if (line.endsWith(":")) {
+			m_mainEditor->insertPlainText("    ");
+		}
+	} else if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE
+			   || m_mode == EDIT_SCO_MODE || m_mode == EDIT_INC_MODE) {
+		linecursor.movePosition(QTextCursor::PreviousBlock);
+		linecursor.select(QTextCursor::LineUnderCursor);
+		QString line = linecursor.selectedText();
+		QRegExp regex = QRegExp("\\s+");
+		if (line.indexOf(regex) == 0) {
+			m_mainEditor->insertPlainText(regex.cap());
+		}
+	}
 }
 
 void DocumentView::findReplace()
@@ -552,28 +935,7 @@ void DocumentView::inToGet()
 	}
 }
 
-void DocumentView::autoComplete()
-{
-	if (m_viewMode < 2) {
-		internalChange = true;
-		QTextCursor cursor = m_mainEditor->textCursor();
-		cursor.select(QTextCursor::WordUnderCursor);
-		QString opcodeName = cursor.selectedText();
-		if (opcodeName=="")
-			return;
-		m_mainEditor->setTextCursor(cursor);
-		m_mainEditor->cut();
-		QString syntax = m_opcodeTree->getSyntax(opcodeName);
-		internalChange = true;
-		m_mainEditor->insertPlainText(syntax);
-	}
-	else { //  Split view
-		// TODO check properly for line number also from other editors
-		qDebug() << "DocumentView::autoComplete() not implemented for split view.";
-	}
-}
-
-void DocumentView::insertTextFromAction()
+void DocumentView::insertAutoCompleteText()
 {
 	TextEditor *editor;
 	if (m_viewMode < 2) {
@@ -582,14 +944,19 @@ void DocumentView::insertTextFromAction()
 	else { //  Split view
 		editor = (TextEditor *) focusWidget();
 		// TODO check properly for line number also from other editors
-		qDebug() << "DocumentView::insertTextFromAction() not implemented for split view.";
+		qDebug() << "DocumentView::insertAutoCompleteText() not implemented for split view.";
 	}
 	if (editor != 0) {
 		internalChange = true;
 		QAction *action = static_cast<QAction *>(QObject::sender());
 		bool insertComplete = static_cast<MySyntaxMenu *>(action->parent())->insertComplete;
+
 		QTextCursor cursor = editor->textCursor();
 		cursor.select(QTextCursor::WordUnderCursor);
+		while ((cursor.selectedText() == "" || cursor.selectedText() == ",") && !cursor.atBlockStart()) {
+			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::MoveAnchor, 2);
+			cursor.select(QTextCursor::WordUnderCursor);
+		}
 		cursor.insertText("");
 		editor->setTextCursor(cursor);
 
@@ -607,10 +974,18 @@ void DocumentView::insertTextFromAction()
 				editor->insertPlainText(syntaxText.mid(index + 1).trimmed());  // right returns the whole string if index < 0
 			}
 			else {
-				editor->insertPlainText(action->data().toString());
+                editor->insertPlainText(action->data().toString());
 			}
-		}
-		else {
+            QString syntaxText = action->data().toString();
+            QStringList syntaxSections = syntaxText.simplified().split("");
+            QString actionText = action->text();
+            actionText = actionText.split(" ").at(0);
+            if (m_autoParameterMode && m_opcodeTree->isOpcode(actionText) && syntaxSections.size() > 1) {
+				m_mainEditor->moveCursor(QTextCursor::StartOfBlock);
+				m_mainEditor->setParameterMode(true);
+				prevParameter();
+			}
+		} else {
 			int index = action->text().indexOf(" ");
 			if (index > 0) {
 				editor->insertPlainText(action->text().left(index));
@@ -619,7 +994,6 @@ void DocumentView::insertTextFromAction()
 				editor->insertPlainText(action->text());
 			}
 		}
-
 	}
 }
 
@@ -627,7 +1001,7 @@ void DocumentView::findString(QString query)
 {
 	// TODO search across all editors
 	if (m_viewMode < 2) {
-		qDebug() << "DocumentView::findString " << query;
+//		qDebug() << "DocumentView::findString " << query;
 		if (query == "") {
 			query = lastSearch;
 		}
@@ -713,7 +1087,6 @@ void DocumentView::createContextMenu(QPoint pos)
 	}
 }
 
-
 void DocumentView::showOrc(bool show)
 {
 	// FIXME set m_viewmode
@@ -872,6 +1245,7 @@ void DocumentView::undo()
 			m_widgetEditor->undo();
 		}
 	}
+	escapePressed();
 }
 
 void DocumentView::redo()
@@ -905,10 +1279,11 @@ void DocumentView::comment()
 	if (m_viewMode < 2) {
 		internalChange = true;
 		QString commentChar = "";
-		if (m_mode == 0) {
+		if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE
+				|| m_mode == EDIT_SCO_MODE || m_mode == EDIT_INC_MODE) {
 			commentChar = ";";
 		}
-		else if (m_mode == 1) { // Python Mode
+		else if (m_mode == EDIT_PYTHON_MODE) { // Python Mode
 			commentChar = "#";
 		}
 		QTextCursor cursor = m_mainEditor->textCursor();
@@ -947,10 +1322,11 @@ void DocumentView::uncomment()
 	if (m_viewMode < 2) {
 		internalChange = true;
 		QString commentChar = "";
-		if (m_mode == 0) {
+		if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE
+				|| m_mode == EDIT_SCO_MODE || m_mode == EDIT_INC_MODE) {
 			commentChar = ";";
 		}
-		else if (m_mode == 1) { // Python Mode
+		else if (m_mode == EDIT_PYTHON_MODE) { // Python Mode
 			commentChar = "#";
 		}
 		QTextCursor cursor = m_mainEditor->textCursor();
@@ -987,20 +1363,22 @@ void DocumentView::indent()
 		//   qDebug("DocumentPage::indent");
 		internalChange = true;
 		QString indentChar = "";
-		if (m_mode == 0) {
+		if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE
+				|| m_mode == EDIT_SCO_MODE || m_mode == EDIT_INC_MODE) {
 			indentChar = "\t";
 		}
-		else if (m_mode == 1) { // Python Mode
+		else if (m_mode == EDIT_PYTHON_MODE) { // Python Mode
 			indentChar = "    ";
 		}
 		QTextCursor cursor = m_mainEditor->textCursor();
+		QTextCursor::MoveMode moveMode = cursor.selectedText().isEmpty() ? QTextCursor::MoveAnchor : QTextCursor::KeepAnchor;
 		if (cursor.position() > cursor.anchor()) {
 			int temp = cursor.anchor();
 			cursor.setPosition(cursor.position());
 			cursor.setPosition(temp, QTextCursor::KeepAnchor);
 		}
 		if (!cursor.atBlockStart()) {
-			cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
+			cursor.movePosition(QTextCursor::StartOfLine, moveMode);
 		}
 		int start = cursor.selectionStart();
 		QString text = cursor.selectedText();
@@ -1011,7 +1389,7 @@ void DocumentView::indent()
 		}
 		cursor.insertText(text);
 		cursor.setPosition(start);
-		cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, text.size());
+		cursor.movePosition(QTextCursor::NextCharacter, moveMode, text.size());
 		m_mainEditor->setTextCursor(cursor);
 	}
 	else {
@@ -1025,10 +1403,11 @@ void DocumentView::unindent()
 	if (m_viewMode < 2) {
 		internalChange = true;
 		QString indentChar = "";
-		if (m_mode == 0) {
+		if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE
+				|| m_mode == EDIT_SCO_MODE || m_mode == EDIT_INC_MODE) {
 			indentChar = "\t";
 		}
-		else if (m_mode == 1) { // Python Mode
+		else if (m_mode == EDIT_PYTHON_MODE) { // Python Mode
 			indentChar = "    ";
 		}
 		QTextCursor cursor = m_mainEditor->textCursor();
@@ -1064,14 +1443,6 @@ void DocumentView::killLine()
 {
 	// TODO implment for multiple views
 	if (m_viewMode < 2) {
-		//  internalChange = true;
-		//    QString indentChar = "";
-		//  if (m_mode == 0) {
-		//    indentChar = "\t";
-		//  }
-		//  else if (m_mode == 1) { // Python Mode
-		//    indentChar = "    ";
-		//  }
 		QTextCursor cursor = m_mainEditor->textCursor();
 		if (!cursor.atBlockStart()) {
 			cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
@@ -1089,13 +1460,6 @@ void DocumentView::killToEnd()
 	// TODO implment for multiple views
 	//  internalChange = true;
 	if (m_viewMode < 2) {
-		QString indentChar = "";
-		//  if (m_mode == 0) {
-		//    indentChar = "\t";
-		//  }
-		//  else if (m_mode == 1) { // Python Mode
-		//    indentChar = "    ";
-		//  }
 		QTextCursor cursor = m_mainEditor->textCursor();
 		if (!cursor.atBlockEnd()) {
 			cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
@@ -1269,11 +1633,6 @@ QString DocumentView::changeToInvalue(QString text)
 	return newText;
 }
 
-//void DocumentView::createSyntaxMenu()
-//{
-//  syntaxMenu->show();
-//}
-
 void DocumentView::destroySyntaxMenu()
 {
 	syntaxMenu->hide();
@@ -1302,19 +1661,32 @@ void MySyntaxMenu::keyPressEvent(QKeyEvent * event)
 		this->hide();
 	}
 	else if (event->key() == Qt::Key_Tab) {
-		insertComplete = false;
 		QAction * a = activeAction();
-		this->hide();
+		insertComplete = false;
+		this->close();
 		if (a != 0) {
 			a->trigger();
 			return;
 		}
 		else {
-			emit keyPressed(event->text());
+			Q_ASSERT(defaultAction() != NULL);
+			defaultAction()->trigger();
+			return;
 		}
-	}
-	else if (event->key() != Qt::Key_Up && event->key() != Qt::Key_Down
-			 && event->key() != Qt::Key_Return) {
+	} else if (event->key() == Qt::Key_Return) {
+		QAction * a = activeAction();
+		insertComplete = true;
+		this->close();
+		if (a != 0) {
+			a->trigger();
+			return;
+		}
+		else {
+			Q_ASSERT(defaultAction() != NULL);
+			defaultAction()->trigger();
+			return;
+		}
+	} else if (event->key() != Qt::Key_Up && event->key() != Qt::Key_Down) {
 		this->close();
 		if (event->key() != Qt::Key_Backspace) {
 			emit keyPressed(event->text());
@@ -1325,6 +1697,7 @@ void MySyntaxMenu::keyPressEvent(QKeyEvent * event)
 				par->event(event);
 		}
 	}
-	insertComplete = true;
+	insertComplete = false;
 	QMenu::keyPressEvent(event);
 }
+
